/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2012-2021, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include "env.h"
#include "core/log.h"


namespace Winix
{


namespace Fun
{

Env::Env()
{
	fun.url = L"env";
	puser   = 0;
}


bool Env::HasAccess()
{
	if( !cur->session->puser )
		return false;

	if( cur->request->IsParam(L"a") )
	{
		// show/change admin environment variables for a user

		if( !cur->session->puser->super_user )
			return false;
	}

return true;
}



bool Env::Parse(const std::wstring & env_str)
{
	space.clear();
	conf_parser.SetSpace(space);

	return (conf_parser.ParseSpace(env_str) == PT::SpaceParser::ok);
}


bool Env::EditAdminEnv(const std::wstring & env_str, bool use_ses_log)
{
	if( Parse(env_str) )
	{
		puser->aenv = space;

		if( puser->update() )
		{
			return true;
		}
		else
		{
			log << log1 << "Evn: a database problem with changing admin environment variables for user: "
				<< puser->name << ", id: " << puser->id << logend;
		}
	}
	else
	{
		log << log2 << "Env: Syntax error in line: " << conf_parser.get_last_parsed_line() << logend;

		if( use_ses_log )
			slog << logerror << T("syntax_error_in_line") << ' ' << conf_parser.get_last_parsed_line() << logend;
	}

return false;
}


bool Env::EditEnv(const std::wstring & env_str, bool use_ses_log)
{
	if( Parse(env_str) )
	{
		puser->env = space;

		if( puser->update() )
		{
			return true;
		}
		else
		{
			log << log1 << "Evn: a database problem with changing admin environment variables for user: "
				<< puser->name << ", id: " << puser->id << logend;
		}
	}
	else
	{
		log << log2 << "Env: Syntax error in line: " << conf_parser.get_last_parsed_line() << logend;

		if( use_ses_log )
			slog << logerror << T("syntax_error_in_line") << ' ' << conf_parser.get_last_parsed_line() << logend;
	}

return false;
}



void Env::SaveEnv()
{
	const std::wstring & env_str = cur->request->PostVar(L"envvar");
	bool status  = false;

	if( cur->request->IsParam(L"a") )
	{
		if( cur->session->puser->super_user )
			status = EditAdminEnv(env_str, true);
	}
	else
	{
		status = EditEnv(env_str, true);
	}

	if( status )
		system->RedirectToLastItem();
}



void Env::MakePost()
{
	puser = nullptr;

	if( cur->session->puser )
	{
		puser = cur->session->puser;

		if( cur->request->IsPostVar(L"changeuser") )
		{
			// show environments variables for the specified user
			if( puser->super_user && cur->request->IsPostVar(L"userid") )
			{
				long id = Tol(cur->request->PostVar(L"userid"));
				puser   = system->users.GetUser(id);

				if( puser )
				{
					log << log2 << "Env: changing user to: " << puser->name << ", id: " << puser->id << logend;
				}
			}
		}

		if( puser )
		{
			/*
			 * this puser should be set in a new struct (based on Model)
			 * and put to templates
			 *
			 */

			SaveEnv();
		}
		else
		{
			cur->request->status = WINIX_ERR_PERMISSION_DENIED;
		}
	}
}


} // namespace


} // namespace Winix

